# Generated by Django 4.2.3 on 2023-10-13 12:40
from django.db import migrations
from guardian.shortcuts import assign_perm

def create_group(apps, project, field_name, permissions):
    Group = apps.get_model("auth", "Group")
    Permission = apps.get_model("auth", "Permission")
    GroupObjectPermission = apps.get_model("guardian", "GroupObjectPermission")
    ContentType = apps.get_model("contenttypes", "ContentType")
    group = Group.objects.create(name=f'LDP_{project._meta.model_name}_{field_name}_{project.id}')
    setattr(project, field_name, group)
    for perm_name in permissions:
        ctype = ContentType.objects.get(model='project')
        permission = Permission.objects.get(content_type=ctype, codename=f'{perm_name}_project')
        GroupObjectPermission.objects.get_or_create(permission=permission, group=group, content_type=ctype, object_pk=project.pk)

def migrate_groups(apps, schema_editor):
    '''copies the members and admins of the projects into new groups'''
    Project = apps.get_model("djangoldp_project", "Project")
    for project in Project.objects.all():
        create_group(apps, project, 'members', ['view'])
        create_group(apps, project, 'admins', ['view', 'change', 'control'])

        for member in project.oldmembers.all():
            project.members.user_set.add(member.user)
            if member.is_admin:
                project.admins.user_set.add(member.user)
        project.save()

def reverse_migrate_groups(apps, schema_editor):
    '''copies the members of the projects into the old member model'''
    Project = apps.get_model("djangoldp_project", "Project")
    Member = apps.get_model("djangoldp_project", "Member")
    for project in Project.objects.all():
        if project.members:
            for user in project.members.user_set.all():
                if not Member.objects.filter(user=user, project=project).exists():
                    is_admin = user in project.admins.user_set.all()
                    Member.objects.create(user=user, project=project, is_admin=is_admin)
            project.members.delete()
        if project.admins:
            project.admins.delete()


def migrate_status(apps, schema_editor):
    '''copies the status of the projects into the public flag'''
    Project = apps.get_model("djangoldp_project", "Project")
    for project in Project.objects.all():
        if project.status == 'Public':
            project.public = True
            project.save()

def reverse_migrate_status(apps, schema_editor):
    '''copies the status of the projects into the public flag'''
    Project = apps.get_model("djangoldp_project", "Project")
    for project in Project.objects.all():
        project.status == 'Public' if project.public else 'Private'


def migrate_communities(apps, schema_editor):
    '''copies the communities from the intermediary model into the inner field'''
    CommunityProject = apps.get_model("djangoldp_community", "CommunityProject")
    for communityproject in CommunityProject.objects.all():
        communityproject.project.community = communityproject.community
        communityproject.project.save()

def reverse_migrate_communities(apps, schema_editor):
    '''copies the communities from the inner field into the intermediary model'''
    Project = apps.get_model("djangoldp_project", "Project")
    CommunityProject = apps.get_model("djangoldp_community", "CommunityProject")
    for project in Project.objects.all():
        if project.community:
            project.oldcommunity, created = CommunityProject.objects.get_or_create(project=project, community=project.community)
            project.save()


class Migration(migrations.Migration):
    dependencies = [
        ('djangoldp_project', '0023_alter_businessprovider_options_and_more'),
    ]

    operations = [
        migrations.RunPython(migrate_status, reverse_migrate_status),
        migrations.RunPython(migrate_groups, reverse_migrate_groups),
        migrations.RunPython(migrate_communities, reverse_migrate_communities),
    ]


# Generated by Django 4.2.3 on 2023-07-13 15:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0021_auto_20210521_1202'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customer',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'ordering': ['pk']},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'ordering': ['pk']},
        ),
    ]



import random
import string
from django.conf import settings
from django.contrib.auth.models import Group
from django.db import models
from django.dispatch import receiver
from django.db.models.signals import pre_save
from djangoldp.models import Model, DynamicNestedField
from djangoldp.permissions import AuthenticatedOnly, AnonymousReadOnly, CreateOnly, ReadAndCreate, \
    PublicPermission, OwnerPermissions, InheritPermissions, ACLPermissions
from djangoldp_account.models import LDPUser
from djangoldp_account.permissions import IPOpenPermissions
from djangoldp_community.models import Community

import logging
logger = logging.getLogger('djangoldp')


class Customer(Model):
    name = models.CharField(max_length=255, null=True, blank=True)
    address = models.TextField(null=True, blank=True)
    postcode = models.CharField(max_length=10, null=True, blank=True)
    city = models.CharField(max_length=50, null=True, blank=True)
    country = models.CharField(max_length=50, null=True, blank=True)
    logo = models.URLField(blank=True, null=True)
    companyRegister = models.CharField(max_length=255, null=True, blank=True)
    first_name = models.CharField(max_length=255, null=True, blank=True)
    last_name = models.CharField(max_length=255, null=True, blank=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="owned_customers", null=True, blank=True, on_delete=models.SET_NULL)
    role = models.CharField(max_length=255, null=True, blank=True)
    email = models.EmailField(null=True, blank=True)
    phone = models.CharField(max_length=255, null=True, blank=True)

    class Meta(Model.Meta):
        ordering = ['pk']
        auto_author = 'owner'
        owner_field = 'owner'
        permission_classes = [AuthenticatedOnly, CreateOnly|OwnerPermissions|InheritPermissions]
        inherit_permissions = ['projects']

    def __str__(self):
        try:
            return '{} ({})'.format(self.name, self.urlid)
        except:
            return self.urlid


def auto_increment_project_number():
  last_inc = Project.objects.all().order_by('id').last()
  if not last_inc:
    return 1
  return last_inc.number + 1


STATUS_CHOICES = [
    ('Public', 'Public'),
    ('Private', 'Private'),
    ('Archived', 'Archived'),
]

project_fields = ["@id", "name", "description", "status", "number", "creationDate", "customer", "captain", "driveID", "businessProvider", "jabberID", "jabberRoom", "members", 'community']


class Project(Model):
    name = models.CharField(max_length=255, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    #TODO: set cascade. For now we allow the deletion of groups while keeping the circles
    community = models.ForeignKey(Community, related_name="projects", on_delete=models.SET_NULL, null=True, blank=True)
    #TODO: remove the status. We keep it for now for the migration
    status = models.CharField(max_length=8, choices=STATUS_CHOICES, default='Private', null=True, blank=True)
    public = models.BooleanField(default=False)
    number = models.PositiveIntegerField(default=auto_increment_project_number, editable=False)
    creationDate = models.DateField(auto_now_add=True)
    customer = models.ForeignKey(Customer, on_delete=models.SET_NULL, related_name='projects', null=True, blank=True)
    captain = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, blank=True, null=True,
                                related_name='+')
    #TODO: set cascade. For now we allow the deletion of groups while keeping the circles
    members = models.OneToOneField(Group, related_name="project", on_delete=models.SET_NULL, null=True, blank=True)
    admins = models.OneToOneField(Group, related_name="admin_project", on_delete=models.SET_NULL, null=True, blank=True)
    driveID = models.TextField(null=True, blank=True)
    jabberID = models.CharField(max_length=255, blank=True, null=True)
    jabberRoom = models.BooleanField(default=True)

    class Meta(Model.Meta):
        ordering = ['pk']
        empty_containers = ["captain"]
        auto_author = 'captain'
        public_field = 'public'
        permission_classes = [IPOpenPermissions|(AuthenticatedOnly&InheritPermissions&(PublicPermission|ACLPermissions))]
        inherit_permissions = ['community']
        permission_roles = {
            'members': {'perms': ['view'], 'add_author': True},
            'admins': {'perms': ['view', 'change', 'control'], 'add_author': True},
        }
        rdf_type = 'hd:project'

    def __str__(self):
        try:
            return '{} {} ({})'.format(self.customer.name, self.name, self.urlid)
        except:
            return self.urlid

    def get_admins(self):
        return self.members.filter(is_admin=True)

# add projects in groups and users
Group._meta.inherit_permissions += ['project','admin_project']
Group._meta.serializer_fields += ['project', 'admin_project']
LDPUser._meta.serializer_fields.append('projects')
LDPUser.projectContainer = lambda self: {'@id': f'{self.urlid}projects/'}
LDPUser._meta.nested_fields.append('projects')
LDPUser.projects = lambda self: Project.objects.filter(members__user=self)
LDPUser.projects.field = DynamicNestedField(Project, 'projects')

class BusinessProvider(Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='businessprovider', null=True, blank=True)
    name = models.CharField(max_length=255, null=True, blank=True)
    fee = models.PositiveIntegerField(default='0', null=True, blank=True)

    class Meta(Model.Meta):
        permission_classes = [AnonymousReadOnly, ReadAndCreate]

    def __str__(self):
        try:
            return '{} ({})'.format(self.name, self.urlid)
        except:
            return self.urlid


#TODO: Remove this model. For now we keep it for the migration
class Member(Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='oldmembers', null=True, blank=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='projects', null=True, blank=True)
    is_admin = models.BooleanField(default=False)

    class Meta(Model.Meta):
        container_path = "project-members/"
        permission_classes = [ACLPermissions]
        unique_together = ['user', 'project']
        rdf_type = 'hd:projectmember'

    def __str__(self):
        try:
            return '{} -> {} ({})'.format(self.project.urlid, self.user.urlid, self.urlid)
        except:
            return self.urlid


@receiver(pre_save, sender=Project)
def set_jabberid(sender, instance, **kwargs):
    if isinstance(getattr(settings, 'JABBER_DEFAULT_HOST', False), str) and not instance.jabberID:
        instance.jabberID = '{}@conference.{}'.format(
            ''.join(
                [
                    random.choice(string.ascii_letters + string.digits)
                    for n in range(12)
                ]
            ).lower(),
            settings.JABBER_DEFAULT_HOST
        )
        instance.jabberRoom = True


# import sys
# import yaml

# import django
# from django.conf import settings as django_settings
# from djangoldp.conf.ldpsettings import LDPSettings
# from djangoldp.tests.server_settings import yaml_config

# # override config loading
# config = {
#     # add the packages to the reference list
#     'ldppackages': ['djangoldp_account', 'djangoldp_community', 'djangoldp_project',
#     #TODO: remove the unused dependencies as soon as they are not necessary in community anymore
#                     'modeltranslation', 'djangoldp_circle', 'djangoldp_joboffer', 'djangoldp_skill', 'djangoldp_conversation', 'djangoldp_notification',
#                     'djangoldp_project.tests'],

#     # required values for server
#     'server': {
#         'SECRET_KEY': "$r&)p-4k@h5b!1yrft6&q%j)_p$lxqh6#)jeeu0z1iag&y&wdu",
#         'AUTH_USER_MODEL': 'djangoldp_account.LDPUser',
#         'REST_FRAMEWORK': {
#             'DEFAULT_PAGINATION_CLASS': 'djangoldp.pagination.LDPPagination',
#             'PAGE_SIZE': 5
#         },
#         # map the config of the core settings (avoid asserts to fail)
#         'SITE_URL': 'http://happy-dev.fr',
#         'BASE_URL': 'http://happy-dev.fr',
#         'SEND_BACKLINKS': False,
#         'JABBER_DEFAULT_HOST': None,
#         'PERMISSIONS_CACHE': False,
#         'SERIALIZER_CACHE': False
#     }
# }
# ldpsettings = LDPSettings(config)
# ldpsettings.config = yaml.safe_load(yaml_config)

# django_settings.configure(ldpsettings)

# django.setup()
# from django.test.runner import DiscoverRunner

# test_runner = DiscoverRunner(verbosity=1)

# failures = test_runner.run_tests([
#     'djangoldp_project.tests.tests_permissions',
# ])
# if failures:
#     sys.exit(failures)



# Generated by Django 4.2.3 on 2023-10-14 17:01

from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0013_auto_20231014_1439'),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('djangoldp_project', '0022_alter_customer_options_alter_project_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='businessprovider',
            options={'default_permissions': {'change', 'add', 'delete', 'control', 'view'}},
        ),
        migrations.AlterModelOptions(
            name='customer',
            options={'default_permissions': {'change', 'add', 'delete', 'control', 'view'}, 'ordering': ['pk']},
        ),
        migrations.AlterModelOptions(
            name='member',
            options={'default_permissions': {'change', 'add', 'delete', 'control', 'view'}},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'default_permissions': {'change', 'add', 'delete', 'control', 'view'}, 'ordering': ['pk']},
        ),
        migrations.AddField(
            model_name='project',
            name='admins',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='admin_project', to='auth.group'),
        ),
        migrations.AddField(
            model_name='project',
            name='community',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='projects', to='djangoldp_community.community'),
        ),
        migrations.AddField(
            model_name='project',
            name='members',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='project', to='auth.group'),
        ),
        migrations.AddField(
            model_name='project',
            name='public',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='businessprovider',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, db_index=True, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, db_index=True, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='member',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='oldmembers', to='djangoldp_project.project'),
        ),
        migrations.AlterField(
            model_name='member',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, db_index=True, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='project',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, db_index=True, null=True, unique=True),
        ),
    ]



from djangoldp.filters import LocalObjectFilterBackend
from djangoldp.views.ldp_viewset import LDPViewSet
from django.http import Http404

class ProjectsJoinableViewset(LDPViewSet):
    filter_backends = [LocalObjectFilterBackend]

    def get_queryset(self):
        return super().get_queryset().exclude(members__user=self.request.user.id)\
            .exclude(status="Private")\
            .exclude(status="Archived")



from django.urls import path
from .views import ProjectsJoinableViewset
from .models import Project
from djangoldp.models import Model


urlpatterns = [
    path('projects/joinable/', ProjectsJoinableViewset.urls(model_prefix="projects-joinable",
        model=Project,
        lookup_field=getattr(Project._meta, 'lookup_field', 'pk'),
        permission_classes=getattr(Project._meta, 'permission_classes', []),
        fields=getattr(Project._meta, 'serializer_fields', []),
        nested_fields=getattr(Project._meta, 'nested_fields', [])))
]



import uuid
import json

from rest_framework.test import APITestCase, APIClient
from djangoldp_project.models import Project, Customer
from djangoldp_account.models import LDPUser


class PermissionsTestCase(APITestCase):
    def setUp(self):
        self.client = APIClient()

    def setUpLoggedInUser(self, is_superuser=False):
        self.user = LDPUser(email='test@mactest.co.uk', first_name='Test', last_name='Mactest', username='test',
                         password='glass onion', is_superuser=is_superuser)
        self.user.save()
        self.client.force_authenticate(user=self.user)

    def _get_random_project(self, public=True, customer=None):
        return Project.objects.create(name='Test', public=public, customer=customer)

    def _get_random_customer(self, owner=None):
        return Customer.objects.create(owner=owner)

    def setUpProject(self, public=True):
        self.project = self._get_random_project(public)

    def _get_request_json(self, **kwargs):
        res = {
            '@context': {
                '@vocab': "https://cdn.startinblox.com/owl#",
                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
                'ldp': "http://www.w3.org/ns/ldp#",
                'foaf': "http://xmlns.com/foaf/0.1/",
                'name': "rdfs:label",
                'acl': "http://www.w3.org/ns/auth/acl#",
                'permissions': "acl:accessControl",
                'mode': "acl:mode",
                'inbox': "https://cdn.startinblox.com/owl#inbox",
                'object': "https://cdn.startinblox.com/owl#object",
                'author': "https://cdn.startinblox.com/owl#author",
                'account': "https://cdn.startinblox.com/owl#account",
                'jabberID': "foaf:jabberID",
                'picture': "foaf:depiction",
                'firstName': "https://cdn.startinblox.com/owl#first_name",
                'lastName': "https://cdn.startinblox.com/owl#last_name",
                'isAdmin': "https://cdn.startinblox.com/owl#is_admin"
            }
        }

        for kwarg in kwargs:
            if isinstance(kwargs[kwarg], str):
                res.update({kwarg: {'@id': kwargs[kwarg]}})
            else:
                res.update({kwarg: kwargs[kwarg]})

        return res

    def _get_random_user(self):
        return LDPUser.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test', last_name='Test',
                                   username=str(uuid.uuid4()))

    # test project permissions
    def test_list_project_anonymous(self):
        self.setUpProject()
        response = self.client.get('/projects/')
        self.assertEqual(response.status_code, 403)

    def test_list_project_authenticated(self):
        self.setUpLoggedInUser()
        # a public project, a private project I'm in and a private project I'm not in
        another_user = self._get_random_user()
        public_project = self._get_random_project()
        my_project = self._get_random_project(public=False)
        my_project.members.user_set.add(self.user)
        private_project = self._get_random_project(public=False)

        response = self.client.get('/projects/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data['ldp:contains']), 2)

    # test customer permissions
    def test_list_customer_anonymous(self):
        self._get_random_customer()
        response = self.client.get('/customers/')
        self.assertEqual(response.status_code, 403)

    def test_list_customer_authenticated(self):
        self.setUpLoggedInUser()
        self._get_random_customer()
        response = self.client.get('/customers/')
        self.assertEqual(response.status_code, 200)

    def test_get_customer_anonymous(self):
        customer = self._get_random_customer()
        response = self.client.get('/customers/{}/'.format(customer.pk))
        self.assertEqual(response.status_code, 403)

    def test_get_customer_authenticated(self):
        self.setUpLoggedInUser()
        customer = self._get_random_customer()
        self._get_random_project(public=False, customer=customer)
        response = self.client.get('/customers/{}/'.format(customer.pk))
        self.assertEqual(response.status_code, 403)

    def test_get_customer_owner(self):
        self.setUpLoggedInUser()
        customer = self._get_random_customer(owner=self.user)
        response = self.client.get('/customers/{}/'.format(customer.pk))
        self.assertEqual(response.status_code, 200)

    # members of one of their projects can view the customer
    def test_get_customer_project_member(self):
        self.setUpLoggedInUser()
        customer = self._get_random_customer()
        project = self._get_random_project(public=False, customer=customer)
        project.members.user_set.add(self.user)
        response = self.client.get('/projects/{}/'.format(project.pk))
        self.assertEqual(response.status_code, 200)
        response = self.client.get('/customers/{}/'.format(customer.pk))
        self.assertEqual(response.status_code, 200)

    def test_post_customer_anonymous(self):
        response = self.client.post('/customers/', data=json.dumps({}), content_type='application/ld+json')
        self.assertEqual(response.status_code, 403)

    def test_post_customer_authenticated(self):
        self.setUpLoggedInUser()
        response = self.client.post('/customers/', data=json.dumps({}), content_type='application/ld+json')
        self.assertEqual(response.status_code, 201)

    # removing a Member - I am an admin
    def test_delete_project_member_admin(self):
        self.setUpLoggedInUser()
        self.setUpProject(public=False)
        self.project.members.user_set.add(self.user)
        self.project.admins.user_set.add(self.user)
        another_user = self._get_random_user()
        self.project.members.user_set.add(another_user)

        self.assertEqual(self.project.members.user_set.count(), 2)
        payload = self._get_request_json(user_set=[{'@id': self.user.urlid}])
        response = self.client.patch(f'/groups/{self.project.members.id}/', json.dumps(payload), content_type='application/ld+json')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.project.members.user_set.count(), 1)